'''
    @author: Peter Pul
    @copyright Joost Broekens, 2008
    When using this, please cite the (Broekens & Brinkman, 2009) ACII Paper.
    This is the AffectButton, a Java button to input emotions.
    The button is based on a well known and widely used emotion theory (see e.g., Mehrabian, 1984, but also many others).
    It has been used in robotics and agents (by e.g., Breazael, 2001 Broekens & DeGroot, 2004 and many others) to express emotions
    It assumes there are three fundamental factors that can be used to describe an emotion (or emotional situation):
    Pleasure - describing how positive/negative the emotion is
    Arousal - describing how active/passive the emotions is
    Dominance - describing how much you feel in charge/dominated
    The main button class, AffectButton, is acting just like any Java button (see example code below)
    the button is scalable (although a very small button will not render the face very well of course, 75*75 being approximately the lowest values that make sense from a user input point of view).
    An extra event, AffectButtonActionEvent, has been introduce that extends the ActionEvent to handle the affective feedback.

    The interaction is as follows: move inside the button and use the mouse wheel
    This version uses the Mousewheel to input the Arousal value.
    It has been evaluated in the (Broekens & Brinkman, 2009) ACII Paper 
'''
from PyQt4 import QtCore, QtGui

from PADFaceMapped import PADFaceMapped

import math

class AffectButton(QtGui.QPushButton):
    '''
    classdocs
    '''
    OS      = 0
    CUSTOM  = 1
    up      = False #was in, but in is an python reserved word
    down    = False

    def __init__(self, parent, face = None):
        QtGui.QWidget.__init__(self,parent)
        if face != None:
            self.resize(face.size, face.size)
        else:
            self.resize(20, 20)
            
        self.face       = face

        self.init()
        
        self.setMouseTracking(1)
        self.setEnabled(1)
        self.show()
    
    def init(self):
        self.osStyle    = True
        self.p          = 0
        self.a          = 0
        self.d          = 0
        self.setCursor(QtCore.Qt.CrossCursor)


#    button functions
    def setStyle (self, style):
        if (style == self.OS):
            self.osStyle    = True
            return
        elif (style == self.CUSTOM): 
            self.osStyle    = False 
            return
        else:
            self.osStyle    = True 
            return

    def getP(self):
        return self.p

    def getA(self):
        return self.a

    def getD(self):
        return self.d
    
    def getLabel(self):
        return self.text()
    
    def setLabel(self, label):
        self.setText(str(label))
    
#    emotion handling and button painting stuff
    def getEmotionId(self, p, a, d):
#        assumes p a and d are between -1 and 1, transforms them to 0-3 then calculates the index with p=high a=medium d=low factor
        if p == 1:
            p = 0.999
        if a == 1:
            a = 0.999
        if d == 1:
            d = 0.999
        return int(((p + 1.0) * 3.0) / 2.0) * 9 + int(((a + 1.0) * 3.0 )/ 2.0) * 3 + int(((d + 1.0) * 3.0 )/ 2.0)
    
    def getEmotionFromId(self, id):
        return self.emotionEvents[id]
    
    def reset(self):
        self.p  = 0
        self.a  = 0
        self.d  = 0
        self.face.setEmotion(self.p, self.a, self.d)
        
    def setXY(self, x, y):
#        x and y between -1 and 1
        factor      = 0.55 #old XXX factor used in acii, affine, mpref: 0.66, now it is 6 to increase responsiveness of arousal
        sensitivity = 1.1 #XXX test with users between 1.1 and 1.2: a sensitivity of 1 means the complete space of the button is used. 2 means only the middle part is used, making it more sensitive and more easy to get to extreme arousal (from a CHI perspective).
        p           = x * sensitivity
        d           = y * sensitivity
#        now do the mapping to arousal.
#        Arousal is controlled in the outer ring of the button.
#        The inner ring is a square of size 66% of the button in which p and d are controlled (see <factor> below).
#        Arousal is controlled in the outer ring (between 66% and the edge of the button).
#        Arousal is based on the position of the mouse pointer in the outer ring.
#        So at the edge, a = 1, and within the 66% of the inner square, a=-1.
        if (p > factor or d > factor or p<-factor or d<-factor):
            if (p==0):
                if (d>0):
                    x1 = 0
                    y1 = factor
                    x2 = 0
                    y2 = 1
                else:
                    x1 = 0
                    y1 = -1 * factor
                    x2 = 0
                    y2 = -1
            else:
                rc = d / p
                if (p >= d and not ((-1 * p) >= d)):
                    x1 = factor
                    x2 = 1
                    y1 = x1*rc
                    y2 = x2*rc
                elif (p >= d and (-1 * p) >= d):
                    y1 = -1 * factor
                    y2 = -1
                    x1 = y1/rc
                    x2 = y2/rc
                elif (not (p >= d) and (-1 * p) >= d):
                    x1 = -1 * factor
                    x2 = -1
                    y1 = x1*rc
                    y2 = x2*rc                     
                else:
                    #(!(p>=d) & !(-p>=d))
                    y1 = factor
                    y2 = 1
                    x1 = y1/rc
                    x2 = y2/rc
            d1  = math.sqrt(math.pow(x1 - x2, 2) + math.pow(y1 - y2, 2))
            a_0 = math.sqrt(math.pow(x1 - p, 2)  + math.pow(y1 - d, 2))
            a   = 2 * (a_0 / d1) - 1
        else:
            a   = -1
#        TEST: not in orginal XXX make p and d behave as if only in factor area, hence p and d completely indepdendent from a, as a is controlled outside, need to check corrs in SAM data and mehrabian
#        p=p/factor;
#        d=d/factor;
        self.p   = 1 if (p > 1) else (-1 if (p < -1) else p)
        self.d   = 1 if (d > 1) else (-1 if (d < -1) else d)
        self.a   = 1 if (a > 1) else (-1 if (a < -1) else a)
    
    def paintEvent(self, event, gg = None):
        if gg == None:
            self.paintEvent(event, QtGui.QPainter(self))
            return

        faceSize        = min((self.width() - 4), (self.height() - 4))
        if (self.face == None):
            self.face   = PADFaceMapped(max(2, (self.width() - faceSize) / 2), max(2, (self.height() - faceSize) / 2), faceSize)

        if (self.osStyle):
            QtGui.QPushButton.paintEvent(self, event)
            self.buffer = QtGui.QPainter(self)
            self.face.setDimensions(max(2, (self.width() - faceSize) / 2), max(2, (self.height() - faceSize) / 2), faceSize)
            self.face.paint(self.buffer)
#            self.buffer.draw_line(0, self.height() -1, self.width() -1, self.height()-1)
        else:
#            this is the CUSTOM way of drawing the AffectButton
            self.buffer      = gg #QtGui.QPainter(self)
            
#            self.buffer.fillRect(0, 0, self.width(), self.height(), self.getBackground())
#            self.buffer.drawRect(0, 0, self.width() - 1, self.height() - 1)
            
            if (self.isActive()):
                self.buffer.setPen(QtGui.QColor(200,200,200))
                self.buffer.drawLine(0, 0, self.width()-1, 0)
                self.buffer.drawLine(0, 0, 0, self.height()-1)
                self.buffer.setPen(QtGui.QColor(125, 125, 125))
                self.buffer.drawLine(self.width() - 1, self.height() - 1, self.width() - 1, 0)
                self.buffer.drawLine(self.width() - 1, self.height() - 1, 0, self.height() - 1)
            if (self.down):
                self.buffer.setPen(QtGui.QColor(125, 125, 125))
                self.buffer.drawLine(0, 0, self.width() - 1, 0)
                self.buffer.drawLine(0, 0, 0, self.height() - 1)
                self.buffer.setPen(QtGui.QColor(200,200,200))
                self.buffer.drawLine(self.width() - 1, self.height() - 1, self.width()-1, 0)
                self.buffer.drawLine(self.width() - 1, self.height() - 1, 0, self.height() - 1)
            
            self.face.setDimensions(max(2, (self.width() - faceSize) / 2), max(2, (self.height() - faceSize) / 2), faceSize)
            self.face.paint(self.buffer)
        
    def isActive(self):
        return self.up
        
#    mouse handling stuff
    def mouseMoveEvent(self, event):
        if (self.isEnabled()):
            self.p   =  (float(event.x()) - float(self.width())  / float(2)) / (float(self.width() + 1) / 2.0)
            self.d   = -(float(event.y()) - float(self.height()) / float(2)) / (float(self.height()+ 1) / 2.0)
            self.setXY(self.p, self.d)
            self.face.setEmotion(self.getP(), self.getA(), self.getD(), event.x(), event.y())
            self.update()

    def mousePressEvent(self, event):
        self.down   = True
        self.update()
    
    def mouseReleaseEvent(self, event):
        self.down   = False
        self.update()

    def enterEvent(self, event):
        self.up     = True
        self.update()
    
    def leaveEvent(self, event):
        self.up     = False
        self.down   = False
        self.update()

        